import numpy as np

def flip_horizontal(image, annotations, image_shape):
    # Ensure image is at least 2D
    if len(image.shape) < 2:
        raise ValueError("Input image must be at least 2D.")

    flipped_image = np.fliplr(image)

    transformed_annotations = []
    for annotation in annotations:
        if len(annotation) != 5:
            raise ValueError("Each annotation should have four elements: [x1, y1, x2, y2]")
        
        label,x1, y1, x2, y2 = annotation
        transformed_x1 = image_shape[0] - x2
        transformed_x2 = image_shape[0] - x1
        
        x_center = (transformed_x1 + transformed_x2) / 2
        x_center_norm=x_center/image_shape[0]
        y_center = (y1 + y2) / 2
        y_center_norm=y_center/image_shape[1]
        
        width = transformed_x2 - transformed_x1
        width_norm=width/image_shape[0]
        height = y2 - y1
        height_norm=height/image_shape[1]
        
        transformed_annotations.append([label,x_center_norm, y_center_norm, width_norm, height_norm])

    return flipped_image, transformed_annotations

def flip_vertical(image, annotations, image_shape):
    # Ensure image is at least 2D
    if len(image.shape) < 2:
        raise ValueError("Input image must be at least 2D.")

    flipped_image = np.flipud(image)

    transformed_annotations = []
    for annotation in annotations:
        if len(annotation) != 5:
            raise ValueError("Each annotation should have four elements: [x1, y1, x2, y2]")
        
        label,x1, y1, x2, y2 = annotation
        transformed_y1 = image_shape[1] - y2
        transformed_y2 = image_shape[1] - y1
        
        x_center = (x1 + x2) / 2
        x_center_norm=x_center/image_shape[0]
        y_center = (transformed_y1 + transformed_y2) / 2
        y_center_norm=y_center/image_shape[1]
        
        
        height = transformed_y2 - transformed_y1
        height_norm=height/image_shape[1]
        width = x2 - x1
        width_norm=width/image_shape[0]
        
        transformed_annotations.append([label,x_center_norm, y_center_norm, width_norm, height_norm])

    return flipped_image, transformed_annotations





import numpy as np

def adjust_brightness(image, factor):
    return np.clip(image * factor, 0, 255)

def adjust_contrast(image, factor):
    mean_value = np.mean(image)
    return np.clip((image - mean_value) * factor + mean_value, 0, 255)



import numpy as np

def rotate(image, angle):
    radians = np.radians(angle)
    rotation_matrix = np.array([[np.cos(radians), -np.sin(radians)],
                                [np.sin(radians), np.cos(radians)]])
    return np.dot(image, rotation_matrix)

def rotate_clockwise(image, angle):
    return np.rot90(image, k=int(angle / 90), axes=(1, 0))

def rotate_anticlockwise(image, angle):
    return np.rot90(image, k=int(-angle / 90), axes=(1, 0))


import numpy as np

def change_color_channels(image, red_factor=1, green_factor=1, blue_factor=1):
    image[:, :, 0] *= red_factor
    image[:, :, 1] *= green_factor
    image[:, :, 2] *= blue_factor
    return np.clip(image, 0, 255)

def intensify_color(image, color, factor):
    color_idx = {'red': 0, 'green': 1, 'blue': 2}
    channel = color_idx[color.lower()]
    image[:, :, channel] *= factor
    return np.clip(image, 0, 255)

def grayscale(image):
    return np.dot(image[..., :3], [0.2989, 0.587, 0.114])



import numpy as np

def random_crop(image, crop_size):
    height, width = image.shape[:2]
    crop_height, crop_width = crop_size
    if height < crop_height or width < crop_width:
        raise ValueError("Crop size must be smaller than image size")
    y = np.random.randint(0, height - crop_height + 1)
    x = np.random.randint(0, width - crop_width + 1)
    return image[y:y+crop_height, x:x+crop_width]



import numpy as np
from scipy.ndimage import convolve

def sharpen(image):
    kernel = np.array([[-1, -1, -1],
                       [-1,  9, -1],
                       [-1, -1, -1]])
    return convolve(image, kernel)

def blur(image):
    kernel = np.array([[1, 1, 1],
                       [1, 1, 1],
                       [1, 1, 1]]) / 9
    return convolve(image, kernel)

def add_noise(image, noise_level):
    noise = np.random.normal(scale=noise_level, size=image.shape)
    return np.clip(image + noise, 0, 255)

def scale_image(image, scale_factor):
    new_height, new_width = int(image.shape[0] * scale_factor), int(image.shape[1] * scale_factor)
    return np.resize(image, (new_height, new_width, image.shape[2]))



from .flip import flip_horizontal, flip_vertical
from .rotate import rotate_clockwise, rotate_anticlockwise
from .random_crop import random_crop
from .color_space import change_color_channels, intensify_color, grayscale
from .kernel_filters import sharpen, blur, add_noise, scale_image
from .resize import resize
from .brightness_contrast import adjust_brightness, adjust_contrast



from setuptools import setup, find_packages
 
setup(
    name='imgaugtools',
    version='1.0.0',
    packages=find_packages(),
    install_requires=[
        # List of dependencies
        "numpy>=1.0",
        "opencv-python>=4.5.1",
    ],
    author='Abhinav Chaturvedi',
    description='easy to use image augmentation tool',
    Long_description='''
    imgaugtool  is a Python package for image augmentation. It provides a collection of tools and utilities for augmenting images and corresponding annotations, helping users enhance their image datasets for machine learning and computer vision tasks.

    Key Features:

    Apply a variety of geometric transformations such as rotation.
    Perform color space transformations including changing RGB color channels and converting to grayscale.
    Apply kernel filters such as sharpening, blurring, and noise addition.
    Support for handling annotations files and transforming coordinates accordingly.
    Simple and easy-to-use tool for applying augmentations to images and annotations.
'''
)


def transform_bbox(annotation, image_shape, transform_func):
    # Transform bounding box coordinates based on image transformation
    transformed_bbox = transform_func(annotation, image_shape)
    
    label,x1, y1, x2, y2 = transformed_bbox
    width = x2 - x1
    width_norm=width/image_shape[0]
    height = y2 - y1
    height_norm=height/image_shape[1]
    x_center = x1 + width / 2
    x_center_norm = x_center/image_shape[0]
    y_center = y1 + height / 2
    y_center_norm = y_center/image_shape[1]
    
    transformed_annotation = f"{label} {x_center_norm} {y_center_norm} {width_norm} {height_norm}"
    
    return transformed_annotation
